
#ifndef _napster_h
#define _napster_h

typedef struct _Stats {
	int libraries;
	int gigs;
	int songs;
	unsigned long total_files;
	double total_filesize;
	unsigned long files_served;
	double filesize_served;
	unsigned long files_received;
	double filesize_received;
	double max_downloadspeed;
	double max_uploadspeed;
	time_t starttime;
	unsigned long shared_files;
	double shared_filesize;
} Stats;

extern Stats statistics;

typedef struct {
	char *username;
	char *password;
	int connection_speed;
} _N_AUTH;

typedef unsigned char _N_CMD;
                
typedef struct {
	int libraries;
	int gigs;
	int songs;
} _N_STATS;

#define cparse convert_output_format

typedef struct _nick_struct {
	struct _nick_struct *next;
	char	*nick;
  	int	speed;
	unsigned long shared;
} NickStruct;

typedef struct _channel_struct {
	struct _channel_struct *next;
	char *channel;
	char *topic;
	int injoin;
	NickStruct *nicks;
} ChannelStruct;

typedef struct _file_struct {
	struct _file_struct *next;
	char *name;
	char *checksum;
	unsigned long filesize;
	unsigned int bitrate;
	unsigned int freq;
	unsigned int seconds;
	char *nick;
	unsigned long ip;
	unsigned short speed;	
} FileStruct;

typedef struct _getfile_ {
	struct _getfile_ *next;	

	char	*nick;
	char	*ip;
	char	*checksum;
	char	*filename;
	char	*realfile;

	int	socket;
	int	port;
	int	write;
	int	count;
	unsigned long filesize;
	unsigned long received;
	unsigned long resume;
	time_t	starttime;
	time_t	addtime;
	int	speed;
	int	up;
} GetFile;


enum nap_Commands {
	CMDR_ERROR		= 0,
	CMDS_UNKNOWN		= 1,
	CMDS_LOGIN		= 2,
	CMDR_EMAILADDR		= 3,

	CMDS_REGISTERINFO	= 6, /* userinfo */
	CMDS_CREATEUSER		= 7, /* create user account */
	CMDR_CREATED		= 8, /* account created */
	CMDR_CREATEERROR	= 9, /* username taken */
#if 0
write(1, "Creating account...\n", 20)   = 20
send(6, "\3\0\7\0qr1", 7, 0)            = 7
recv(6, "\0\0\10\0", 4, 0)              = 4
write(1, "Registered username\n", 20)   = 20
send(6, "3\0\6\0qr1 qr1 6699 \"nap v0.6\" 4 sabina@pagan.vampires.net", 55, 0) = 55
#endif
	CMDR_LOGINERROR		= 13, 
	CMDR_MSTAT		= 15,
	CMDR_REQUESTUSERSPEED	= 89,
	CMDR_SENDFILE		= 95,
	CMDS_ADDFILE		= 100,
	CMDR_GETQUEUE		= 108,
	CMDR_MOTD		= 109,
	CMDR_ANOTHERUSER	= 148,
	CMDS_SEARCH		= 200,
	CMDR_SEARCHRESULTS	= 201,
	CMDR_SEARCHRESULTSEND	= 202,
	CMDS_REQUESTFILE	= 203,
	CMDR_FILEREADY		= 204,
	CMDS_SENDMSG		= 205,
	CMDR_GETERROR		= 206,
	CMDS_ADDHOTLIST		= 208,
	CMDR_HOTLISTONLINE	= 209,
	CMDR_USEROFFLINE	= 210, 
	CMDS_BROWSE		= 211,
	CMDR_BROWSERESULT	= 212,
	CMDR_BROWSEENDRESULT	= 213,
	CMDR_STATS		= 214,

	CMDR_HOTLISTOFFLINE	= 303, /* ?? */
	
	CMDS_JOIN		= 400,
	CMDS_PART		= 401,
	CMDS_SEND		= 402,

	CMDR_PUBLIC		= 403,
	CMDR_ERRORMSG		= 404,
	CMDR_JOIN		= 405,
	CMDR_JOINNEW		= 406,
	CMDR_PARTED		= 407,
	CMDR_NAMES		= 408,
	CMDR_ENDNAMES		= 409,
	CMDS_TOPIC		= 410, /* got/change topic */

	CMDS_FILEINFOREQ	= 500,
	CMDR_FILEINFO		= 501,

	/* 601 and 602 */
	CMDS_REQUESTINFO	= 600,
	CMDS_FILESIZE		= 601,
	CMDS_REQUESTSIZE	= 602,
	
	
	CMDS_WHOIS		= 603,
	CMDR_WHOIS		= 604,
	CMDR_WHOWAS		= 605,
	CMDS_SETUSERLEVEL	= 606, /* moderators/administrators/elite */

	CMDR_FILEREQUEST	= 607, /* nick \"filename\" */
	CMDS_FILEINFO		= 608, /* nick \"filename\" */

	CMDS_CANCELFILE		= 609,
	
	CMDS_KILLUSER		= 610, /* return 404 permission denied */
	CMDS_NUKEUSER		= 611, /* return 404 */
	CMDS_BANUSER		= 612,
	CMDS_SETDATAPORT	= 613, 
	CMDS_UNBANUSER		= 614,
	CMDS_BANLIST		= 615,
	CMDR_BANLIST		= 616,
	CMDS_LISTCHANNELS	= 617,
	CMDR_LISTCHANNELS	= 618,

	CMDR_GETFILEERROR	= 620, /* FILE queue ? */

	CMDR_MOTDS		= 621, 
	CMDS_MUZZLE		= 622,
	CMDS_UNMUZZLE		= 623,
	CMDS_UNNUKEUSER		= 624, /* return 404 */
	CMDS_SETLINESPEED	= 625,
	CMDR_DATAPORTERROR	= 626,
	CMDS_OPSAY		= 627,
	CMDS_ANNOUNCE		= 628,
	CMDS_CHANGEPASS		= 701
	/* 303 500 501 700 702 703 800 801 802 */
/*	404 error msg */
};

/* 
 500 nick filename  -> "501 nick ip port filename md5 speed"
*/

#define NAP_send(s, i) write(nap_socket, s, i);

typedef struct {
	unsigned short len;
	unsigned short command;
} _N_DATA;

SocketList *naplink_connect(char *, unsigned short);
void naplink_getserver(char *, unsigned short, int);

typedef struct {
	int	cmd;
	int	(*func)(int, char *);
} _NAP_COMMANDS;

#define NAP_COMM(name) \
	int name (int cmd, char *args)

NAP_COMM(cmd_error);
NAP_COMM(cmd_unknown);
NAP_COMM(cmd_login);
NAP_COMM(cmd_email);
NAP_COMM(cmd_stats);
NAP_COMM(cmd_whois);
NAP_COMM(cmd_whowas);
NAP_COMM(cmd_joined);
NAP_COMM(cmd_topic);
NAP_COMM(cmd_names);
NAP_COMM(cmd_endnames);
NAP_COMM(cmd_parted);
NAP_COMM(cmd_public);
NAP_COMM(cmd_msg);
NAP_COMM(cmd_search);
NAP_COMM(cmd_endsearch);
NAP_COMM(cmd_browse);
NAP_COMM(cmd_endbrowse);
NAP_COMM(cmd_request);
NAP_COMM(cmd_getfile);
NAP_COMM(cmd_getfileinfo);
NAP_COMM(cmd_alreadyregistered);
NAP_COMM(cmd_registerinfo);
NAP_COMM(cmd_banlist);
NAP_COMM(cmd_offline);
NAP_COMM(cmd_filerequest);
NAP_COMM(cmd_dataport);
NAP_COMM(cmd_hotlist);
NAP_COMM(cmd_fileinfo);

int nap_say(char *, ...);
int nap_put(char *, ...);
int send_ncommand(unsigned int, char *, ...);

void load_napserv(IrcCommandDll *, char *, char *, char *, char *);
void print_napster(IrcCommandDll *, char *, char *, char *, char *);
void share_napster(IrcCommandDll *, char *, char *, char *, char *);
void nap_del(IrcCommandDll *, char *, char *, char *, char *);
void nap_glist(IrcCommandDll *, char *, char *, char *, char *);
void nap_request(IrcCommandDll *, char *, char *, char *, char *);
void print_file(FileStruct *, int);
void naplink_handlelink(int);
void make_listen(void);
void set_napster_socket(int);
char *napster_status(void);
int build_napster_status(Window *);
int nap_finished_file(int, GetFile *);
int clean_queue(GetFile **, int);

GetFile *find_in_getfile(GetFile **, int, char *, char *, char *, int, int);

char *base_name(char *);

extern FileStruct *file_search;
extern FileStruct *file_browse;
extern GetFile *napster_sendqueue;
extern char *nap_current_channel;
extern int nap_socket;

#define NAP_DOWNLOAD 0
#define NAP_UPLOAD 1
#define NAP_BUFFER_SIZE 2048

#endif
